@@ -9,7 +9,6 @@ from zipfile import ZipFile
 import numpy as np
 import pandas as pd
 import panel as pn
-import pylems_py2xml
 import lems.api as lems
 
 # import local packages
@@ -117,8 +116,6 @@ def main(path: str, files: list, subs: dict = None, save: bool = False, layout:
             # if results:
             #     convert.IGNORE_CENTRE = results
 
-    print(subs)
-
     # only save conversions if 'save' is True
     if save and subs:
         save_output(subs)
@@ -139,11 +136,11 @@ def main(path: str, files: list, subs: dict = None, save: bool = False, layout:
         check_output()
         pn.state.notifications.success(f'{OUTPUT} folder is ready!')
 
-    if H5_CONTENT is not None and 'model' in H5_CONTENT.keys():
-        pylems_py2xml.main.XML(inp=H5_CONTENT, output_path=os.path.join(OUTPUT, 'param'),
-                               uid=H5_CONTENT['model'], app=True, suffix=DESC)
-        MODEL_NAME = utils.get_model()
-        transfer_xml()
+    # if H5_CONTENT is not None and 'model' in H5_CONTENT.keys():
+    #     pylems_py2xml.main.XML(inp=H5_CONTENT, output_path=os.path.join(OUTPUT, 'param'),
+    #                            uid=H5_CONTENT['model'], app=True, suffix=DESC)
+    #     MODEL_NAME = utils.get_model()
+    #     transfer_xml()
 
     # finally, remove all empty folders
     remove_empty()
